@@ -1,15 +1,4 @@
-use std::fs;
-use std::io;
-use std::path::{Path, PathBuf};
-
-use csv_index::RandomAccessSimple;
-
-use crate::config::{Config, Delimiter};
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Creates an index of the given CSV data, which can make other operations like
 slicing, splitting and gathering statistics much faster.
 
@@ -35,15 +24,22 @@ index options:
 
 Common options:
     -h, --help             Display this message
-    -d, --delimiter <arg>  The field delimiter for reading CSV data.
-                           Must be a single character. (default: ,)
-";
+"#;
+
+use std::{
+    fs, io,
+    path::{Path, PathBuf},
+};
+
+use csv_index::RandomAccessSimple;
+use serde::Deserialize;
+
+use crate::{config::Config, util, CliResult};
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input: String,
+    arg_input:   String,
     flag_output: Option<String>,
-    flag_delimiter: Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -54,9 +50,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         Some(p) => PathBuf::from(&p),
     };
 
-    let rconfig = Config::new(&Some(args.arg_input)).delimiter(args.flag_delimiter);
+    let rconfig = Config::new(&Some(args.arg_input)).checkutf8(false);
     let mut rdr = rconfig.reader_file()?;
-    let mut wtr = io::BufWriter::new(fs::File::create(&pidx)?);
+    let mut wtr = io::BufWriter::new(fs::File::create(pidx)?);
     RandomAccessSimple::create(&mut rdr, &mut wtr)?;
     Ok(())
 }
