@@ -1,15 +1,4 @@
-use std::io;
-
-use rand::{self, rngs::StdRng, seq::SliceRandom, Rng, SeedableRng};
-
-use crate::config::{Config, Delimiter};
-use crate::index::Indexed;
-use crate::util;
-use crate::CliResult;
-use log::debug;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Randomly samples CSV data uniformly using memory proportional to the size of
 the sample.
 
@@ -27,6 +16,8 @@ once, which is necessary to provide a uniform random sample. If you wish to
 limit the number of records visited, use the 'qsv slice' command to pipe into
 'qsv sample'.
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_sample.rs.
+
 Usage:
     qsv sample [options] <sample-size> [<input>]
     qsv sample --help
@@ -43,27 +34,41 @@ Common options:
                            in the output.)
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-";
+"#;
+
+use std::io;
+
+use log::debug;
+use rand::{self, rngs::StdRng, seq::SliceRandom, Rng, SeedableRng};
+use serde::Deserialize;
+
+use crate::{
+    config::{Config, Delimiter},
+    index::Indexed,
+    util, CliResult,
+};
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input: Option<String>,
+    arg_input:       Option<String>,
     arg_sample_size: f64,
-    flag_output: Option<String>,
+    flag_output:     Option<String>,
     flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
-    flag_seed: Option<usize>,
+    flag_delimiter:  Option<Delimiter>,
+    flag_seed:       Option<usize>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
+        .checkutf8(false)
         .no_headers(args.flag_no_headers);
     let mut sample_size = args.arg_sample_size;
 
     let mut wtr = Config::new(&args.flag_output).writer()?;
     let sampled = if let Some(mut idx) = rconfig.indexed()? {
+        #[allow(clippy::cast_precision_loss)]
         if sample_size < 1.0 {
             sample_size *= idx.count() as f64;
         }
@@ -104,7 +109,7 @@ where
     let mut rng = ::rand::thread_rng();
     // this non-cryptographic shuffle is sufficient for our use case
     // as we're optimizing for performance
-    SliceRandom::shuffle(&mut *all_indices, &mut rng);
+    SliceRandom::shuffle(&mut *all_indices, &mut rng); //DevSkim: ignore DS148264
 
     let mut sampled = Vec::with_capacity(sample_size as usize);
     for i in all_indices.into_iter().take(sample_size as usize) {
@@ -133,7 +138,7 @@ fn sample_reservoir<R: io::Read>(
         None => StdRng::from_rng(rand::thread_rng()).unwrap(),
         // the non-cryptographic seed_from_u64 is sufficient for our use case
         // as we're optimizing for performance
-        Some(seed) => StdRng::seed_from_u64(seed as u64),
+        Some(seed) => StdRng::seed_from_u64(seed as u64), //DevSkim: ignore DS148264
     };
 
     // Now do the sampling.
@@ -147,7 +152,7 @@ fn sample_reservoir<R: io::Read>(
 }
 
 fn do_random_access(sample_size: u64, total: u64) -> bool {
-    let raflag = sample_size <= (total / 10) as u64;
+    let raflag = sample_size <= (total / 10);
     debug!("sample_size: {sample_size}, total: {total}, raflag: {raflag}");
     raflag
 }
