@@ -1,30 +1,8 @@
-use crate::cmd::stats::Stats;
-use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
-use crate::util;
-use crate::CliError;
-use crate::CliResult;
-use ahash::AHashMap;
-use csv::ByteRecord;
-use grex::RegExpBuilder;
-use itertools::Itertools;
-use log::{debug, error, info, warn};
-use serde::Deserialize;
-use serde_json::{json, value::Number, Map, Value};
-use stats::Frequencies;
-use std::{collections::HashSet, fs::File, io::Write, path::Path};
-
-macro_rules! fail {
-    ($mesg:expr) => {
-        return Err(CliError::Other($mesg));
-    };
-}
-
 static USAGE: &str = r#"
 Generate JSON Schema from CSV data.
 
 This command generates reference JSON Schema (Draft 7) from CSV data, 
-including validation rules based on input data range.
+including validation rules based on data type and input data domain/range.
 
 Running `validate` command on original input CSV with generated schema 
 should not flag any invalid records.
@@ -36,8 +14,11 @@ a `stdin.csv` file will created with stdin's contents as well.
 
 Note that `stdin.csv` will be overwritten if it already exists.
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_schema.rs.
+
 Usage:
     qsv schema [options] [<input>]
+    qsv schema --help
 
 Schema options:
     --enum-threshold NUM       Cardinality threshold for adding enum
@@ -51,7 +32,7 @@ Schema options:
                                it is shortlisted for date inferencing.
                                Set to "all" to inspect ALL fields for
                                date/datetime types.
-                               [default: date,time,due,opened,closed]
+                               [default: date,time,due,open,close,created]
     --prefer-dmy               Prefer to parse dates in dmy format. Otherwise, use mdy format.
     --stdout                   Send generated JSON schema file to stdout instead.
     -j, --jobs <arg>           The number of jobs to run in parallel.
@@ -68,18 +49,36 @@ Common options:
                                Must be a single character. [default: ,]
 "#;
 
+use std::{collections::HashSet, fs::File, io::Write, path::Path};
+
+use ahash::AHashMap;
+use csv::ByteRecord;
+use grex::RegExpBuilder;
+use itertools::Itertools;
+use log::{debug, error, info, warn};
+use serde::Deserialize;
+use serde_json::{json, value::Number, Map, Value};
+use stats::Frequencies;
+
+use crate::{
+    cmd::stats::Stats,
+    config::{Config, Delimiter},
+    select::SelectColumns,
+    util, CliResult,
+};
+
 #[derive(Deserialize, Clone)]
-struct Args {
-    flag_enum_threshold: usize,
-    flag_strict_dates: bool,
-    flag_pattern_columns: SelectColumns,
-    flag_dates_whitelist: String,
-    flag_prefer_dmy: bool,
-    flag_stdout: bool,
-    flag_jobs: Option<usize>,
-    flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
-    arg_input: Option<String>,
+pub struct Args {
+    pub flag_enum_threshold:  usize,
+    pub flag_strict_dates:    bool,
+    pub flag_pattern_columns: SelectColumns,
+    pub flag_dates_whitelist: String,
+    pub flag_prefer_dmy:      bool,
+    pub flag_stdout:          bool,
+    pub flag_jobs:            Option<usize>,
+    pub flag_no_headers:      bool,
+    pub flag_delimiter:       Option<Delimiter>,
+    pub arg_input:            Option<String>,
 }
 
 const STDIN_CSV: &str = "stdin.csv";
@@ -115,8 +114,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         match infer_schema_from_stats(&args, &input_filename) {
             Ok(map) => map,
             Err(e) => {
-                let msg = format!("Failed to infer schema via stats and frequency: {e}");
-                fail!(msg);
+                return fail_clierror!("Failed to infer schema via stats and frequency: {e}");
             }
         };
 
@@ -146,48 +144,41 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         "required": Value::Array(required_fields)
     });
 
-    let schema_pretty = serde_json::to_string_pretty(&schema).expect("prettify schema json");
+    let schema_pretty = match serde_json::to_string_pretty(&schema) {
+        Ok(s) => s,
+        Err(e) => return fail_clierror!("Cannot prettify schema json: {e}"),
+    };
 
     if args.flag_stdout {
         let stdout = std::io::stdout();
         let mut handle = stdout.lock();
 
-        handle
-            .write_all(schema_pretty.as_bytes())
-            .expect("unable to write schema file to stdout");
-
-        handle.flush().unwrap();
+        handle.write_all(schema_pretty.as_bytes())?;
+        handle.flush()?;
 
         info!("Schema written to stdout");
     } else {
         let schema_output_filename = input_path + ".schema.json";
-        let mut schema_output_file =
-            File::create(&schema_output_filename).expect("unable to create schema output file");
+        let mut schema_output_file = File::create(&schema_output_filename)?;
 
-        schema_output_file
-            .write_all(schema_pretty.as_bytes())
-            .expect("unable to write schema file");
+        schema_output_file.write_all(schema_pretty.as_bytes())?;
+        schema_output_file.flush()?;
 
-        // flush error report; file gets closed automagically when out-of-scope
-        schema_output_file.flush().unwrap();
-
-        println!("Schema written to {schema_output_filename}");
-        info!("Schema written to {schema_output_filename}");
+        woutinfo!("Schema written to {schema_output_filename}");
     }
 
     Ok(())
 }
 
-/// Builds JSON MAP object that corresponds to the "properties" object of JSON Schema (Draft 7) by looking at CSV value stats
-/// Supported JSON Schema validation vocabularies:
+/// Builds JSON MAP object that corresponds to the "properties" object of JSON Schema (Draft 7) by
+/// looking at CSV value stats Supported JSON Schema validation vocabularies:
 ///  * type
 ///  * enum
 ///  * minLength
 ///  * maxLength
 ///  * min
 ///  * max
-#[allow(clippy::len_zero)]
-fn infer_schema_from_stats(args: &Args, input_filename: &str) -> CliResult<Map<String, Value>> {
+pub fn infer_schema_from_stats(args: &Args, input_filename: &str) -> CliResult<Map<String, Value>> {
     // invoke cmd::stats
     let (csv_fields, csv_stats, stats_col_index_map) = get_stats_records(args)?;
 
@@ -222,16 +213,19 @@ fn infer_schema_from_stats(args: &Args, input_filename: &str) -> CliResult<Map<S
         let header_string = convert_to_string(header_byte_slice)?;
 
         // grab stats record for current column
-        let stats_record = csv_stats.get(i).unwrap().clone().to_record();
+        let stats_record = csv_stats.get(i).unwrap().clone().to_record(4);
 
-        debug!("stats[{header_string}]: {stats_record:?}");
+        if log::log_enabled!(log::Level::Debug) {
+            debug!("stats[{header_string}]: {stats_record:?}");
+        }
 
         // get Type from stats record
         let col_type = stats_record.get(stats_col_index_map["type"]).unwrap();
         // get NullCount
-        let col_null_count = match stats_record.get(stats_col_index_map["nullcount"]) {
-            Some(s) => s.parse::<usize>().unwrap_or(0_usize),
-            None => 0_usize,
+        let col_null_count = if let Some(s) = stats_record.get(stats_col_index_map["nullcount"]) {
+            s.parse::<usize>().unwrap_or(0_usize)
+        } else {
+            0_usize
         };
 
         // debug!(
@@ -277,20 +271,6 @@ fn infer_schema_from_stats(args: &Args, input_filename: &str) -> CliResult<Map<S
                     }
                 }
             }
-            "Date" => {
-                type_list.push(Value::String("string".to_string()));
-
-                if args.flag_strict_dates {
-                    field_map.insert("format".to_string(), Value::String("date".to_string()));
-                }
-            }
-            "DateTime" => {
-                type_list.push(Value::String("string".to_string()));
-
-                if args.flag_strict_dates {
-                    field_map.insert("format".to_string(), Value::String("date-time".to_string()));
-                }
-            }
             "Integer" => {
                 type_list.push(Value::String("integer".to_string()));
 
@@ -326,6 +306,20 @@ fn infer_schema_from_stats(args: &Args, input_filename: &str) -> CliResult<Map<S
             "NULL" => {
                 type_list.push(Value::String("null".to_string()));
             }
+            "Date" => {
+                type_list.push(Value::String("string".to_string()));
+
+                if args.flag_strict_dates {
+                    field_map.insert("format".to_string(), Value::String("date".to_string()));
+                }
+            }
+            "DateTime" => {
+                type_list.push(Value::String("string".to_string()));
+
+                if args.flag_strict_dates {
+                    field_map.insert("format".to_string(), Value::String("date-time".to_string()));
+                }
+            }
             _ => {
                 warn!("Stats gave unexpected field type '{col_type}', default to JSON String.");
                 // defaults to JSON String
@@ -339,17 +333,17 @@ fn infer_schema_from_stats(args: &Args, input_filename: &str) -> CliResult<Map<S
             type_list.push(Value::String("null".to_string()));
         }
 
-        if col_null_count > 0 && enum_list.len() > 0 {
-            // for fields that are not mandatory and actualy have enum list generated,
+        if col_null_count > 0 && !enum_list.is_empty() {
+            // for fields that are not mandatory and actually have enum list generated,
             // having JSON NULL indicates that missing value is allowed
             enum_list.push(Value::Null);
         }
 
-        if type_list.len() > 0 {
+        if !type_list.is_empty() {
             field_map.insert("type".to_string(), Value::Array(type_list.clone()));
         }
 
-        if enum_list.len() > 0 {
+        if !enum_list.is_empty() {
             field_map.insert("enum".to_string(), Value::Array(enum_list.clone()));
         }
 
@@ -360,38 +354,39 @@ fn infer_schema_from_stats(args: &Args, input_filename: &str) -> CliResult<Map<S
     Ok(properties_map)
 }
 
-/// get stats records from cmd::stats
-/// returns tuple (csv_fields, csv_stats, stats_col_index_map)
+/// get stats records from `cmd::stats`
+/// returns tuple (`csv_fields`, `csv_stats`, `stats_col_index_map`)
 fn get_stats_records(args: &Args) -> CliResult<(ByteRecord, Vec<Stats>, AHashMap<String, usize>)> {
     let stats_args = crate::cmd::stats::Args {
-        arg_input: args.arg_input.clone(),
-        flag_select: crate::select::SelectColumns::parse("").unwrap(),
-        flag_everything: false,
-        flag_mode: false,
-        flag_cardinality: true,
-        flag_median: false,
-        flag_quartiles: false,
-        flag_nulls: false,
-        flag_infer_dates: true,
+        arg_input:            args.arg_input.clone(),
+        flag_select:          crate::select::SelectColumns::parse("").unwrap(),
+        flag_everything:      false,
+        flag_typesonly:       false,
+        flag_mode:            false,
+        flag_cardinality:     true,
+        flag_median:          false,
+        flag_quartiles:       false,
+        flag_nulls:           false,
+        flag_round:           4,
+        flag_infer_dates:     true,
         flag_dates_whitelist: args.flag_dates_whitelist.to_string(),
-        flag_prefer_dmy: args.flag_prefer_dmy,
-        flag_jobs: Some(util::njobs(args.flag_jobs)),
-        flag_output: None,
-        flag_no_headers: args.flag_no_headers,
-        flag_delimiter: args.flag_delimiter,
+        flag_prefer_dmy:      args.flag_prefer_dmy,
+        flag_jobs:            Some(util::njobs(args.flag_jobs)),
+        flag_output:          None,
+        flag_no_headers:      args.flag_no_headers,
+        flag_delimiter:       args.flag_delimiter,
     };
 
     let (csv_fields, csv_stats) = match stats_args.rconfig().indexed() {
-        Ok(o) => match o {
-            None => {
+        Ok(o) => {
+            if let Some(idx) = o {
+                info!("has index, triggering parallel stats");
+                stats_args.parallel_stats(&stats_args.flag_dates_whitelist, &idx)
+            } else {
                 info!("no index, triggering sequential stats");
                 stats_args.sequential_stats(&stats_args.flag_dates_whitelist)
             }
-            Some(idx) => {
-                info!("has index, triggering parallel stats");
-                stats_args.parallel_stats(&stats_args.flag_dates_whitelist, idx)
-            }
-        },
+        }
         Err(e) => {
             warn!("error determining if indexed, triggering sequential stats: {e}");
             stats_args.sequential_stats(&stats_args.flag_dates_whitelist)
@@ -426,7 +421,7 @@ fn build_low_cardinality_column_selector_arg(
     // identify low cardinality columns
     for i in 0..csv_fields.len() {
         // grab stats record for current column
-        let stats_record = csv_stats.get(i).unwrap().clone().to_record();
+        let stats_record = csv_stats.get(i).unwrap().clone().to_record(4);
 
         // get Cardinality
         let col_cardinality = match stats_record.get(stats_col_index_map["cardinality"]) {
@@ -451,23 +446,23 @@ fn build_low_cardinality_column_selector_arg(
     column_select_arg
 }
 
-/// get frequency tables from cmd::stats
-/// returns map of unique valules keyed by header
+/// get frequency tables from `cmd::stats`
+/// returns map of unique values keyed by header
 fn get_unique_values(
     args: &Args,
     column_select_arg: &str,
 ) -> CliResult<AHashMap<String, Vec<String>>> {
     // prepare arg for invoking cmd::frequency
     let freq_args = crate::cmd::frequency::Args {
-        arg_input: args.arg_input.clone(),
-        flag_select: crate::select::SelectColumns::parse(column_select_arg).unwrap(),
-        flag_limit: args.flag_enum_threshold,
-        flag_asc: false,
-        flag_no_nulls: true,
-        flag_jobs: Some(util::njobs(args.flag_jobs)),
-        flag_output: None,
+        arg_input:       args.arg_input.clone(),
+        flag_select:     crate::select::SelectColumns::parse(column_select_arg).unwrap(),
+        flag_limit:      args.flag_enum_threshold,
+        flag_asc:        false,
+        flag_no_nulls:   true,
+        flag_jobs:       Some(util::njobs(args.flag_jobs)),
+        flag_output:     None,
         flag_no_headers: args.flag_no_headers,
-        flag_delimiter: args.flag_delimiter,
+        flag_delimiter:  args.flag_delimiter,
     };
 
     let (headers, ftables) = match freq_args.rconfig().indexed()? {
@@ -501,11 +496,14 @@ fn construct_map_of_unique_values(
         // sort the values so enum list so schema can be diff'ed between runs
         unique_values.sort_unstable();
 
-        debug!(
-            "enum[{header_string}]: len={}, val={:?}",
-            unique_values.len(),
-            unique_values
-        );
+        if log::log_enabled!(log::Level::Debug) {
+            // we do this as this debug is relatively expensive
+            debug!(
+                "enum[{header_string}]: len={}, val={:?}",
+                unique_values.len(),
+                unique_values
+            );
+        }
         unique_values_map.insert(header_string, unique_values);
     }
 
@@ -518,18 +516,16 @@ fn construct_map_of_unique_values(
 #[inline]
 fn convert_to_string(byte_slice: &[u8]) -> CliResult<String> {
     // convert csv header to string
-    let string: String = match std::str::from_utf8(byte_slice) {
-        Ok(s) => s.to_string(),
+    match std::str::from_utf8(byte_slice) {
+        Ok(s) => Ok(s.to_string()),
         Err(e) => {
             let lossy_string = String::from_utf8_lossy(byte_slice);
-            let msg =
-                format!("Can't convert byte slice to utf8 string. slice={byte_slice:?}, error={e}: {lossy_string}");
-            error!("{msg}");
-            fail!(msg);
+            fail_clierror!(
+                "Can't convert byte slice to utf8 string. slice={byte_slice:?}, error={e}: \
+                 {lossy_string}"
+            )
         }
-    };
-
-    Ok(string)
+    }
 }
 
 /// determine required fields
@@ -565,7 +561,8 @@ fn generate_string_patterns(
 
     // return empty pattern map when:
     //  * no columns are selected
-    //  * all columns are selected (by default, all columns are selected when no columns are explicitly specified)
+    //  * all columns are selected (by default, all columns are selected when no columns are
+    //    explicitly specified)
     if sel.len() == 0 || sel.len() == headers.len() {
         debug!("no pattern columns selected");
         return Ok(pattern_map);
@@ -613,7 +610,7 @@ fn generate_string_patterns(
             .with_minimum_repetitions(2)
             .build();
 
-        pattern_map.insert(header.to_owned(), regexp);
+        pattern_map.insert(header.clone(), regexp);
     }
 
     debug!("pattern map: {pattern_map:?}");
@@ -624,7 +621,7 @@ fn generate_string_patterns(
 // only emit "pattern" constraint for String fields without enum constraint
 fn should_emit_pattern_constraint(field_def: &Value) -> bool {
     let type_list = field_def[&"type"].as_array().unwrap();
-    let has_enum = field_def.get(&"enum").is_some();
+    let has_enum = field_def.get("enum").is_some();
 
     type_list.contains(&Value::String("string".to_string())) && !has_enum
 }
