@@ -1,12 +1,4 @@
-use std::fs;
-
-use crate::config::{Config, Delimiter};
-use crate::index::Indexed;
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Returns the rows in the range specified (starting at 0, half-open interval).
 The range does not include headers.
 
@@ -23,6 +15,7 @@ parsed.
 
 Usage:
     qsv slice [options] [<input>]
+    qsv slice --help
 
 slice options:
     -s, --start <arg>      The index of the record to slice from.
@@ -40,18 +33,28 @@ Common options:
                            appear in the output as the header row.
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-";
+"#;
+
+use std::fs;
+
+use serde::Deserialize;
+
+use crate::{
+    config::{Config, Delimiter},
+    index::Indexed,
+    util, CliResult,
+};
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input: Option<String>,
-    flag_start: Option<isize>,
-    flag_end: Option<usize>,
-    flag_len: Option<usize>,
-    flag_index: Option<usize>,
-    flag_output: Option<String>,
+    arg_input:       Option<String>,
+    flag_start:      Option<isize>,
+    flag_end:        Option<usize>,
+    flag_len:        Option<usize>,
+    flag_index:      Option<usize>,
+    flag_output:     Option<String>,
     flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
+    flag_delimiter:  Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -108,6 +111,7 @@ impl Args {
 
     fn rconfig(&self) -> Config {
         Config::new(&self.arg_input)
+            .checkutf8(false)
             .delimiter(self.flag_delimiter)
             .no_headers(self.flag_no_headers)
     }
