@@ -1,16 +1,9 @@
-use std::cmp;
+static USAGE: &str = r#"
+Sorts CSV data in alphabetical (with case-insensitive option), numerical,
+reverse, unique or random (with optional seed) order.
 
-use self::Number::{Float, Int};
-use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
-use crate::util;
-use crate::CliResult;
-use rand::{rngs::StdRng, seq::SliceRandom, SeedableRng};
-use rayon::prelude::*;
-use serde::Deserialize;
-
-static USAGE: &str = "
-Sorts CSV data lexicographically.
+The sort is done in lexicographical order.
+https://en.wikipedia.org/wiki/Lexicographic_order
 
 Note that this requires reading all of the CSV data into memory. If
 you need to sort a large file that may not fit into memory, use the
@@ -18,44 +11,61 @@ extsort command instead.
 
 Usage:
     qsv sort [options] [<input>]
+    qsv sort --help
 
 sort options:
-    -s, --select <arg>     Select a subset of columns to sort.
-                           See 'qsv select --help' for the format details.
-    -N, --numeric          Compare according to string numerical value
-    -R, --reverse          Reverse order
-    --random               Random order
-    --seed <number>        RNG seed
-    -j, --jobs <arg>       The number of jobs to run in parallel.
-                           When not set, the number of jobs is set to the
-                           number of CPUs detected.
+    -s, --select <arg>      Select a subset of columns to sort.
+                            See 'qsv select --help' for the format details.
+    -N, --numeric           Compare according to string numerical value
+    -R, --reverse           Reverse order
+    --random                Random order
+    --seed <number>         RNG seed
+    -i, --ignore-case       Compare strings disregarding case
+    -u, --unique            When set, identical consecutive lines will be dropped
+                            to keep only one line per sorted value.
+    -j, --jobs <arg>        The number of jobs to run in parallel.
+                            When not set, the number of jobs is set to the
+                            number of CPUs detected.
 
 Common options:
-    -h, --help             Display this message
-    -o, --output <file>    Write output to <file> instead of stdout.
-    -n, --no-headers       When set, the first row will not be interpreted
-                           as headers. Namely, it will be sorted with the rest
-                           of the rows. Otherwise, the first row will always
-                           appear as the header row in the output.
-    -d, --delimiter <arg>  The field delimiter for reading CSV data.
-                           Must be a single character. (default: ,)
-    -u, --uniq             When set, identical consecutive lines will be dropped
-                           to keep only one line per sorted value.
-";
+    -h, --help              Display this message
+    -o, --output <file>     Write output to <file> instead of stdout.
+    -n, --no-headers        When set, the first row will not be interpreted
+                            as headers. Namely, it will be sorted with the rest
+                            of the rows. Otherwise, the first row will always
+                            appear as the header row in the output.
+    -d, --delimiter <arg>   The field delimiter for reading CSV data.
+                            Must be a single character. (default: ,)
+"#;
+
+use std::cmp;
+
+use rand::{rngs::StdRng, seq::SliceRandom, SeedableRng};
+use rayon::prelude::*;
+use serde::Deserialize;
+
+use self::Number::{Float, Int};
+use crate::{
+    cmd::dedup::iter_cmp_ignore_case,
+    config::{Config, Delimiter},
+    select::SelectColumns,
+    util, CliResult,
+};
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input: Option<String>,
-    flag_select: SelectColumns,
-    flag_numeric: bool,
-    flag_reverse: bool,
-    flag_random: bool,
-    flag_seed: Option<u64>,
-    flag_jobs: Option<usize>,
-    flag_output: Option<String>,
-    flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
-    flag_uniq: bool,
+    arg_input:        Option<String>,
+    flag_select:      SelectColumns,
+    flag_numeric:     bool,
+    flag_reverse:     bool,
+    flag_random:      bool,
+    flag_seed:        Option<u64>,
+    flag_ignore_case: bool,
+    flag_jobs:        Option<usize>,
+    flag_output:      Option<String>,
+    flag_no_headers:  bool,
+    flag_delimiter:   Option<Delimiter>,
+    flag_unique:      bool,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -66,6 +76,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers)
+        .checkutf8(false)
         .select(args.flag_select);
 
     let mut rdr = rconfig.reader()?;
@@ -78,22 +89,28 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // Seeding rng
     let seed = args.flag_seed;
 
+    let ignore_case = args.flag_ignore_case;
+
     let mut all = rdr.byte_records().collect::<Result<Vec<_>, _>>()?;
     match (numeric, reverse, random) {
         (_, _, true) => {
             // we don't need cryptographically strong RNGs for this
             if let Some(val) = seed {
-                let mut rng = StdRng::seed_from_u64(val);
-                SliceRandom::shuffle(&mut *all, &mut rng);
+                let mut rng = StdRng::seed_from_u64(val); //DevSkim: ignore DS148264
+                SliceRandom::shuffle(&mut *all, &mut rng); //DevSkim: ignore DS148264
             } else {
                 let mut rng = ::rand::thread_rng();
-                SliceRandom::shuffle(&mut *all, &mut rng);
+                SliceRandom::shuffle(&mut *all, &mut rng); //DevSkim: ignore DS148264
             }
         }
         (false, false, false) => all.par_sort_unstable_by(|r1, r2| {
             let a = sel.select(r1);
             let b = sel.select(r2);
-            iter_cmp(a, b)
+            if ignore_case {
+                iter_cmp_ignore_case(a, b)
+            } else {
+                iter_cmp(a, b)
+            }
         }),
         (true, false, false) => all.par_sort_unstable_by(|r1, r2| {
             let a = sel.select(r1);
@@ -103,7 +120,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         (false, true, false) => all.par_sort_unstable_by(|r1, r2| {
             let a = sel.select(r1);
             let b = sel.select(r2);
-            iter_cmp(b, a)
+            if ignore_case {
+                iter_cmp_ignore_case(b, a)
+            } else {
+                iter_cmp(b, a)
+            }
         }),
         (true, true, false) => all.par_sort_unstable_by(|r1, r2| {
             let a = sel.select(r1);
@@ -116,7 +137,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut prev: Option<csv::ByteRecord> = None;
     rconfig.write_headers(&mut rdr, &mut wtr)?;
     for r in all {
-        if args.flag_uniq {
+        if args.flag_unique {
             match prev {
                 Some(other_r) => match iter_cmp(sel.select(&r), sel.select(&other_r)) {
                     cmp::Ordering::Equal => (),
@@ -188,7 +209,9 @@ enum Number {
 fn compare_num(n1: Number, n2: Number) -> cmp::Ordering {
     match (n1, n2) {
         (Int(i1), Int(i2)) => i1.cmp(&i2),
+        #[allow(clippy::cast_precision_loss)]
         (Int(i1), Float(f2)) => compare_float(i1 as f64, f2),
+        #[allow(clippy::cast_precision_loss)]
         (Float(f1), Int(i2)) => compare_float(f1, i2 as f64),
         (Float(f1), Float(f2)) => compare_float(f1, f2),
     }
