@@ -1,18 +1,19 @@
-use std::borrow::ToOwned;
-use std::env;
-use std::fs;
-use std::io::{self, Read};
-use std::ops::Deref;
-use std::path::PathBuf;
+use std::{
+    borrow::ToOwned,
+    env, fs,
+    io::{self, Read},
+    path::{Path, PathBuf},
+};
 
 use log::{debug, info, warn};
 use qsv_sniffer::{SampleSize, Sniffer};
 use serde::de::{Deserialize, Deserializer, Error};
 
-use crate::index::Indexed;
-use crate::select::{SelectColumns, Selection};
-use crate::util;
-use crate::CliResult;
+use crate::{
+    index::Indexed,
+    select::{SelectColumns, Selection},
+    util, CliResult,
+};
 
 // rdr default is 8k in csv crate, we're doubling it
 const DEFAULT_RDR_BUFFER_CAPACITY: usize = 16 * (1 << 10);
@@ -43,16 +44,14 @@ impl Delimiter {
         }
 
         if s.len() != 1 {
-            return Err(format!(
-                "Could not convert '{s}' to a single ASCII character."
-            ));
+            return fail_format!("Could not convert '{s}' to a single ASCII character.");
         }
 
         let c = s.chars().next().unwrap();
         if c.is_ascii() {
             Ok(Delimiter(c as u8))
         } else {
-            Err(format!("Could not convert '{c}' to ASCII delimiter."))
+            fail_format!("Could not convert '{c}' to ASCII delimiter.")
         }
     }
 }
@@ -67,25 +66,26 @@ impl<'de> Deserialize<'de> for Delimiter {
     }
 }
 
+#[allow(clippy::struct_excessive_bools)]
 #[derive(Debug)]
 pub struct Config {
-    path: Option<PathBuf>, // None implies <stdin>
-    idx_path: Option<PathBuf>,
-    select_columns: Option<SelectColumns>,
-    delimiter: u8,
-    pub no_headers: bool,
-    flexible: bool,
-    terminator: csv::Terminator,
-    pub quote: u8,
-    quote_style: csv::QuoteStyle,
-    double_quote: bool,
-    escape: Option<u8>,
-    quoting: bool,
+    path:              Option<PathBuf>, // None implies <stdin>
+    idx_path:          Option<PathBuf>,
+    select_columns:    Option<SelectColumns>,
+    delimiter:         u8,
+    pub no_headers:    bool,
+    flexible:          bool,
+    terminator:        csv::Terminator,
+    pub quote:         u8,
+    quote_style:       csv::QuoteStyle,
+    double_quote:      bool,
+    escape:            Option<u8>,
+    quoting:           bool,
     pub preamble_rows: u64,
-    trim: csv::Trim,
-    autoindex: bool,
-    checkutf8: bool,
-    prefer_dmy: bool,
+    trim:              csv::Trim,
+    autoindex:         bool,
+    checkutf8:         bool,
+    prefer_dmy:        bool,
 }
 
 // Empty trait as an alias for Seek and Read that avoids auto trait errors
@@ -100,7 +100,7 @@ impl Config {
         };
         let (path, mut delim) = match *path {
             None => (None, default_delim),
-            Some(ref s) if s.deref() == "-" => (None, default_delim),
+            Some(ref s) if &**s == "-" => (None, default_delim),
             Some(ref s) => {
                 let path = PathBuf::from(s);
                 let file_extension = path
@@ -198,6 +198,7 @@ impl Config {
         self
     }
 
+    #[cfg(any(feature = "full", feature = "lite"))]
     pub const fn crlf(mut self, yes: bool) -> Config {
         if yes {
             self.terminator = csv::Terminator::CRLF;
@@ -207,6 +208,7 @@ impl Config {
         self
     }
 
+    #[cfg(any(feature = "full", feature = "lite"))]
     pub const fn terminator(mut self, term: csv::Terminator) -> Config {
         self.terminator = term;
         self
@@ -217,6 +219,7 @@ impl Config {
         self
     }
 
+    #[cfg(any(feature = "full", feature = "lite"))]
     pub const fn quote_style(mut self, style: csv::QuoteStyle) -> Config {
         self.quote_style = style;
         self
@@ -242,6 +245,7 @@ impl Config {
         self
     }
 
+    #[allow(clippy::missing_const_for_fn)]
     pub fn select(mut self, sel_cols: SelectColumns) -> Config {
         self.select_columns = Some(sel_cols);
         self
@@ -258,9 +262,7 @@ impl Config {
 
     pub fn selection(&self, first_record: &csv::ByteRecord) -> Result<Selection, String> {
         match self.select_columns {
-            None => Err("Config has no 'SelectColumns'. Did you call \
-                         Config::select?"
-                .to_owned()),
+            None => fail!("Config has no 'SelectColumns'. Did you call Config::select?"),
             Some(ref sel) => sel.selection(first_record, !self.no_headers),
         }
     }
@@ -315,7 +317,18 @@ impl Config {
                 // check if its utf8-encoded
                 if self.checkutf8 {
                     debug!("checking stdin encoding...");
-                    let s = std::str::from_utf8(&buffer);
+                    // get first 8k of buffer
+                    if buffer.is_empty() {
+                        return Err(io::Error::new(
+                            io::ErrorKind::InvalidData,
+                            "<stdin> is empty!".to_string(),
+                        ));
+                    }
+                    let buffer_check = buffer
+                        .chunks_exact(std::cmp::min(DEFAULT_UTF8_CHECK_BUFFER_LEN, buffer.len()))
+                        .next()
+                        .unwrap();
+                    let s = std::str::from_utf8(buffer_check);
                     if s.is_err() {
                         return Err(io::Error::new(
                             io::ErrorKind::InvalidData,
@@ -349,17 +362,13 @@ impl Config {
             let mut f = match fs::File::open(path_buf) {
                 Ok(x) => x,
                 Err(err) => {
-                    let msg = format!("failed to open {}: {}", path_buf.display(), err);
+                    let msg = format!("failed to open {}: {err}", path_buf.display());
                     return Err(io::Error::new(io::ErrorKind::NotFound, msg));
                 }
             };
 
             let fsize = f.metadata().unwrap().len() as usize;
-            let mut buffer_size = DEFAULT_UTF8_CHECK_BUFFER_LEN;
-            if fsize < buffer_size {
-                buffer_size = fsize;
-            }
-            let mut buffer = vec![0; buffer_size];
+            let mut buffer = vec![0; std::cmp::min(DEFAULT_UTF8_CHECK_BUFFER_LEN, fsize)];
             if f.read_exact(&mut buffer).is_ok() {
                 let s = std::str::from_utf8(&buffer);
                 return Ok(s.is_ok());
@@ -369,12 +378,17 @@ impl Config {
     }
 
     fn autoindex_file(&self) {
-        if let Some(path_buf) = &self.path {
-            let path_clone = path_buf.clone();
-            let path_str = path_clone.into_os_string().into_string().unwrap();
-            let index_argv: Vec<&str> = vec!["", "index", &path_str];
-            crate::cmd::index::run(&*index_argv).unwrap();
-            debug!("autoindex for {path_str} created");
+        // autoindex should never panic. It should silently fail as its a "convenience fn"
+        // that's why we have a lot of let-else returns, in lieu of unwraps
+        let Some(path_buf) = &self.path else { return };
+        let Ok(mut rdr) = self.reader_file() else { return };
+
+        let pidx = util::idx_path(Path::new(path_buf));
+        let Ok(idxfile) = fs::File::create(pidx) else { return };
+        let mut wtr = io::BufWriter::new(idxfile);
+        match csv_index::RandomAccessSimple::create(&mut rdr, &mut wtr) {
+            Ok(_) => debug!("autoindex of {path_buf:?} successful."),
+            Err(e) => debug!("autoindex of {path_buf:?} failed: {e}"),
         }
     }
 
@@ -387,10 +401,10 @@ impl Config {
                     "Cannot use <stdin> with indexes",
                 ));
             }
-            (&Some(ref p), &None) => {
+            (Some(p), &None) => {
                 // We generally don't want to report an error here, since we're
                 // passively trying to find an index, so we just log the warning...
-                let idx_file = match fs::File::open(&util::idx_path(p)) {
+                let idx_file = match fs::File::open(util::idx_path(p)) {
                     Err(e) => {
                         if self.autoindex {
                             self.autoindex_file();
@@ -403,7 +417,7 @@ impl Config {
                 };
                 (fs::File::open(p)?, idx_file)
             }
-            (&Some(ref p), &Some(ref ip)) => (fs::File::open(p)?, fs::File::open(ip)?),
+            (Some(p), Some(ip)) => (fs::File::open(p)?, fs::File::open(ip)?),
         };
         // If the CSV data was last modified after the index file was last
         // modified, then return an error and demand the user regenerate the
@@ -417,8 +431,7 @@ impl Config {
             } else {
                 return Err(io::Error::new(
                     io::ErrorKind::Other,
-                    "The CSV file was modified after the index file. \
-                 Please re-create the index.",
+                    "The CSV file was modified after the index file. Please re-create the index.",
                 ));
             }
         }
@@ -440,8 +453,18 @@ impl Config {
                     let stdin_reader = io::stdin();
                     let mut buffer: Vec<u8> = Vec::new();
                     stdin_reader.lock().read_to_end(&mut buffer)?;
+                    if buffer.is_empty() {
+                        return Err(io::Error::new(
+                            io::ErrorKind::InvalidData,
+                            "<stdin> is empty!".to_string(),
+                        ));
+                    }
                     // check if its utf8-encoded
-                    let s = std::str::from_utf8(&buffer);
+                    let buffer_check = buffer
+                        .chunks_exact(std::cmp::min(DEFAULT_UTF8_CHECK_BUFFER_LEN, buffer.len()))
+                        .next()
+                        .unwrap();
+                    let s = std::str::from_utf8(buffer_check);
                     if s.is_err() {
                         return Err(io::Error::new(
                             io::ErrorKind::InvalidData,
@@ -463,7 +486,7 @@ impl Config {
                 match fs::File::open(p) {
                     Ok(x) => Box::new(x),
                     Err(err) => {
-                        let msg = format!("failed to open {}: {}", p.display(), err);
+                        let msg = format!("failed to open {}: {err}", p.display());
                         return Err(io::Error::new(io::ErrorKind::NotFound, msg));
                     }
                 }
@@ -497,7 +520,15 @@ impl Config {
     pub fn io_writer(&self) -> io::Result<Box<dyn io::Write + 'static>> {
         Ok(match self.path {
             None => Box::new(io::stdout()),
-            Some(ref p) => Box::new(fs::File::create(p)?),
+            Some(ref p) => {
+                let p_str = p.as_os_str();
+                if p_str == "sink" {
+                    // sink is /dev/null
+                    Box::new(io::sink())
+                } else {
+                    Box::new(fs::File::create(p)?)
+                }
+            }
         })
     }
 
