@@ -16,9 +16,9 @@ fn index_outdated_count() {
         ],
     );
 
-    let md = fs::metadata(&wrk.path("in.csv.idx")).unwrap();
+    let md = fs::metadata(wrk.path("in.csv.idx")).unwrap();
     set_file_times(
-        &wrk.path("in.csv"),
+        wrk.path("in.csv"),
         future_time(FileTime::from_last_modification_time(&md)),
         future_time(FileTime::from_last_access_time(&md)),
     )
@@ -47,9 +47,9 @@ fn index_outdated_stats() {
         ],
     );
 
-    let md = fs::metadata(&wrk.path("in.csv.idx")).unwrap();
+    let md = fs::metadata(wrk.path("in.csv.idx")).unwrap();
     set_file_times(
-        &wrk.path("in.csv"),
+        wrk.path("in.csv"),
         future_time(FileTime::from_last_modification_time(&md)),
         future_time(FileTime::from_last_access_time(&md)),
     )
@@ -57,11 +57,45 @@ fn index_outdated_stats() {
 
     // stats should fail if the index is stale
     let mut cmd = wrk.command("stats");
-    cmd.arg("in.csv");
+    cmd.env_clear().arg("in.csv");
 
     wrk.assert_err(&mut cmd);
 }
 
+#[test]
+fn index_outdated_stats_autoindex() {
+    let wrk = Workdir::new("index_outdated_stats");
+
+    wrk.create_indexed(
+        "in.csv",
+        vec![
+            svec!["letter", "number"],
+            svec!["a", "1"],
+            svec!["b", "2"],
+            svec!["c", "3"],
+        ],
+    );
+
+    std::env::set_var("QSV_AUTOINDEX", "1");
+
+    let md = fs::metadata(wrk.path("in.csv.idx")).unwrap();
+    set_file_times(
+        wrk.path("in.csv"),
+        future_time(FileTime::from_last_modification_time(&md)),
+        future_time(FileTime::from_last_access_time(&md)),
+    )
+    .unwrap();
+
+    // stats should NOT fail if the index is stale and
+    // QSV_AUTOINDEX is set
+    let mut cmd = wrk.command("stats");
+    cmd.arg("in.csv");
+
+    wrk.assert_success(&mut cmd);
+
+    std::env::remove_var("QSV_AUTOINDEX");
+}
+
 fn future_time(ft: FileTime) -> FileTime {
     let secs = ft.unix_seconds();
     FileTime::from_unix_time(secs + 10_000, 0)
