@@ -1,6 +1,4 @@
-use std::borrow::ToOwned;
-use std::cmp;
-use std::process;
+use std::{borrow::ToOwned, cmp, process};
 
 use crate::workdir::Workdir;
 
@@ -190,7 +188,7 @@ fn get_field_value(wrk: &Workdir, cmd: &mut process::Command, field: &str) -> St
     let mut rows: Vec<Vec<String>> = wrk.read_stdout(cmd);
     let headers = rows.remove(0);
     let mut sequence: Vec<&str> = vec![];
-    for row in rows.iter() {
+    for row in &rows {
         for (h, val) in headers.iter().zip(row.iter()) {
             match field {
                 "quartiles" => match &**h {
@@ -213,8 +211,7 @@ fn get_field_value(wrk: &Workdir, cmd: &mut process::Command, field: &str) -> St
         }
     }
     panic!(
-        "BUG: Could not find field '{}' in headers '{:?}' \
-            for command '{:?}'.",
+        "BUG: Could not find field '{}' in headers '{:?}' for command '{:?}'.",
         field, headers, cmd
     );
 }
@@ -411,6 +408,19 @@ stats_tests!(
     ],
     "OVERFLOW"
 );
+stats_tests!(
+    stats_sum_negative_overflow,
+    "sum",
+    &[
+        {
+            let i = i64::MIN;
+            &i.to_string()
+        },
+        "-1",
+        "-2"
+    ],
+    "UNDERFLOW"
+);
 
 stats_tests!(stats_min, "min", &["2", "1.1"], "1.1");
 stats_tests!(stats_max, "max", &["2", "1.1"], "2");
@@ -425,20 +435,10 @@ stats_tests!(stats_len_min_null, "min_length", &["", "aa", "a"], "0");
 stats_tests!(stats_len_max_null, "max_length", &["a", "aa", ""], "2");
 
 stats_tests!(stats_mean, "mean", &["5", "15", "10"], "10");
-stats_tests!(
-    stats_stddev,
-    "stddev",
-    &["1", "2", "3"],
-    "0.816496580927726"
-);
+stats_tests!(stats_stddev, "stddev", &["1", "2", "3"], "0.8165");
 stats_tests!(stats_variance, "variance", &["3", "5", "7", "9", "11"], "8");
 stats_tests!(stats_mean_null, "mean", &["", "5", "15", "10"], "10");
-stats_tests!(
-    stats_stddev_null,
-    "stddev",
-    &["1", "2", "3", ""],
-    "0.816496580927726"
-);
+stats_tests!(stats_stddev_null, "stddev", &["1", "2", "3", ""], "0.8165");
 stats_tests!(
     stats_variance_null,
     "variance",
@@ -446,12 +446,7 @@ stats_tests!(
     "6"
 );
 stats_tests!(stats_mean_mix, "mean", &["5", "15.1", "9.9"], "10");
-stats_tests!(
-    stats_stddev_mix,
-    "stddev",
-    &["1", "2.1", "2.9"],
-    "0.7788880963698614"
-);
+stats_tests!(stats_stddev_mix, "stddev", &["1", "2.1", "2.9"], "0.7789");
 stats_tests!(
     stats_variance_mix,
     "variance",
@@ -476,31 +471,31 @@ stats_tests!(
     stats_quartiles,
     "quartiles",
     &["1", "2", "3"],
-    "-5.0,-2.0,1.0,2.0,3.0,2.0,6.0,9.0"
+    "-5,-2,1,2,3,2,6,9"
 );
 stats_tests!(
     stats_quartiles_null,
     "quartiles",
     &["", "1", "2", "3"],
-    "-5.0,-2.0,1.0,2.0,3.0,2.0,6.0,9.0"
+    "-5,-2,1,2,3,2,6,9"
 );
 stats_tests!(
     stats_quartiles_even,
     "quartiles",
     &["1", "2", "3", "4"],
-    "-4.5,-1.5,1.5,2.5,3.5,2.0,6.5,9.5"
+    "-4.5,-1.5,1.5,2.5,3.5,2,6.5,9.5"
 );
 stats_tests!(
     stats_quartiles_even_null,
     "quartiles",
     &["", "1", "2", "3", "4"],
-    "-4.5,-1.5,1.5,2.5,3.5,2.0,6.5,9.5"
+    "-4.5,-1.5,1.5,2.5,3.5,2,6.5,9.5"
 );
 stats_tests!(
     stats_quartiles_mix,
     "quartiles",
     &["1", "2.0", "3", "4"],
-    "-4.5,-1.5,1.5,2.5,3.5,2.0,6.5,9.5"
+    "-4.5,-1.5,1.5,2.5,3.5,2,6.5,9.5"
 );
 stats_tests!(stats_quartiles_null_empty, "quartiles", &[""], "");
 
@@ -583,6 +578,23 @@ fn stats_prefer_mdy() {
     assert_eq!(got, expected.replace("\r\n", "\n").trim_end());
 }
 
+#[test]
+fn stats_rounding() {
+    let wrk = Workdir::new("stats_prefer_mdy");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--everything")
+        .args(["--round", "8"])
+        .arg(test_file);
+
+    let got: String = wrk.stdout(&mut cmd);
+
+    let expected = wrk.load_test_resource("boston311-100-everything-8places-stats.csv");
+
+    assert_eq!(got, expected.replace("\r\n", "\n").trim_end());
+}
+
 #[test]
 fn stats_no_date_inference() {
     let wrk = Workdir::new("stats_no_date_inference");
@@ -617,6 +629,50 @@ fn stats_with_date_inference() {
     assert_eq!(got, expected.replace("\r\n", "\n").trim_end());
 }
 
+#[test]
+fn stats_typesonly() {
+    let wrk = Workdir::new("stats_typesonly");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--typesonly").arg(test_file);
+
+    let got: String = wrk.stdout(&mut cmd);
+
+    let expected = wrk.load_test_resource("boston311-100-typesonly-stats.csv");
+
+    assert_eq!(got, expected.replace("\r\n", "\n").trim_end());
+}
+
+#[test]
+fn stats_leading_zero_handling() {
+    let wrk = Workdir::new("stats");
+
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["col1", "col2", "col3"],
+            svec!["1", "4321", "01"],
+            svec!["2", "3210", "02"],
+            svec!["3", "2101", "03"],
+            svec!["4", "1012", "04"],
+            svec!["5", "0", "10"],
+        ],
+    );
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--typesonly").arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["field", "type"],
+        svec!["col1", "Integer"],
+        svec!["col2", "Integer"],
+        svec!["col3", "String"],
+    ];
+    assert_eq!(got, expected);
+}
+
 mod stats_infer_nothing {
     // Only test CSV data with headers.
     // Empty CSV data with no headers won't produce any statistical analysis.
